<?php

// Written at Louisiana State University

require_once($CFG->dirroot . '/blocks/bclnemail/lib.php');

class block_bclnemail extends block_list {
    function init() {
        $this->title = bclnemail::_s('pluginname');
    }

    function applicable_formats() {
        global $USER;
        if(is_siteadmin($USER->id) || has_capability('block/bclnemail:myaddinstance', context_system::instance())) {
            return array('site' => true, 'my' => true, 'course-view' => true);
        } else {
            return array('site' => false, 'my' => false, 'course-view' => true);
        }
    }
    function has_config() {
        return true;
    }
    /**
     * Disable multiple instances of this block
     * @return bool Returns false
     */
    function instance_allow_multiple() {
        return false;
    }
    
    function get_content() {
        global $USER, $CFG, $COURSE, $OUTPUT;

        if ($this->content !== NULL) {
            return $this->content;
        }

        $this->content = new stdClass;
        $this->content->items = array();
        $this->content->icons = array();
        $this->content->footer = '';

        $context = context_course::instance($COURSE->id);

        $config = bclnemail::load_config($COURSE->id);
        $permission = has_capability('block/bclnemail:cansend', $context);

        $can_send = ($permission or !empty($config['allowstudents']));

        $icon_class = array('class' => 'icon');

        $cparam = array('courseid' => $COURSE->id);

        if ($can_send && $COURSE->id != SITEID) {
            $send_email_str = bclnemail::_s('composenew');
            $icon = $OUTPUT->pix_icon('t/email', $send_email_str, 'moodle', $icon_class);
            $send_email = html_writer::link(
                new moodle_url('/blocks/bclnemail/email.php', $cparam),
                $icon.$send_email_str
            );
            $this->content->items[] = $send_email;

            $signature_str = bclnemail::_s('signature');
            $icon = $OUTPUT->pix_icon('i/edit', $signature_str, 'moodle', $icon_class);
            $signature = html_writer::link(
                new moodle_url('/blocks/bclnemail/signature.php', $cparam),
                $icon.$signature_str
            );
            $this->content->items[] = $signature;

            $draft_params = $cparam + array('type' => 'drafts');
            $drafts_email_str = bclnemail::_s('drafts');
            $icon = $OUTPUT->pix_icon('i/settings', $drafts_email_str, 'moodle', $icon_class);
            $drafts = html_writer::link(
                new moodle_url('/blocks/bclnemail/emaillog.php', $draft_params),
                $icon.$drafts_email_str
            );
            $this->content->items[] = $drafts;

            $history_str = bclnemail::_s('history');
            $icon = $OUTPUT->pix_icon('i/settings', $history_str, 'moodle', $icon_class);
            $history = html_writer::link(
                new moodle_url('/blocks/bclnemail/emaillog.php', $cparam),
                $icon.$history_str
            );
            $this->content->items[] = $history;

            
            if (has_capability('block/bclnemail:canconfig', $context)) {
            $config_str = bclnemail::_s('config');
            $icon = $OUTPUT->pix_icon('i/settings', $config_str, 'moodle', $icon_class);
            $config = html_writer::link(
                new moodle_url('/blocks/bclnemail/config_qm.php', $cparam),
                $icon.$config_str
            );
            $this->content->items[] = $config;
        }


        }

        if((has_capability('block/bclnemail:myaddinstance', context_system::instance()) || is_siteadmin($USER->id)) && $COURSE->id == SITEID) {
            $send_adminemail_str = bclnemail::_s('sendadmin');
            $icon = $OUTPUT->pix_icon('t/email', $send_adminemail_str, 'moodle', $icon_class);
            $send_adminemail = html_writer::link(
                new moodle_url('/blocks/bclnemail/admin_email.php'),
                $icon.$send_adminemail_str
            );
            $this->content->items[] = $send_adminemail;
        } 
        if (is_siteadmin($USER->id) && $COURSE->id == SITEID) {
            $history_str = bclnemail::_s('history');
            $icon = $OUTPUT->pix_icon('i/settings', $history_str, 'moodle', $icon_class);
            $history = html_writer::link(
                new moodle_url('/blocks/bclnemail/emaillog.php', $cparam),
                $icon.$history_str
            );
            $this->content->items[] = $history;
        }


        return $this->content;
    }
}
