<?php
//
require_once('../../config.php');
require_once('../../enrol/externallib.php');
require_once('lib.php');
require_once('email_form.php');
require_once('../../lib/weblib.php');

require_login();

ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

$courseid = required_param('courseid', PARAM_INT);
$type = optional_param('type', '', PARAM_ALPHA);
$typeid = optional_param('typeid', 0, PARAM_INT);
$sigid = optional_param('sigid', 0, PARAM_INT);
$messageIDresend = optional_param('fmid', 0, PARAM_INT);

if (!$course = $DB->get_record('course', array('id' => $courseid))) {
    print_error('no_course', 'block_bclnemail', '', $courseid);
}

if (!empty($type) and !in_array($type, array('log', 'drafts'))) {
    print_error('no_type', 'block_bclnemail', '', $type);
}

if (!empty($type) and empty($typeid)) {
    $string = new stdclass;
    $string->tpe = $type;
    $string->id = $typeid;

    print_error('no_typeid', 'block_bclnemail', '', $string);
}

$config = bclnemail::load_config($courseid);

$context = context_course::instance($courseid);
$has_permission = (
        has_capability('block/bclnemail:cansend', $context) or
        !empty($config['allowstudents'])
        );

if (!$has_permission) {
    print_error('no_permission', 'block_bclnemail');
}

$sigs = $DB->get_records('block_bclnemail_signatures', array('userid' => $USER->id), 'default_flag DESC');

$alt_params = array('courseid' => $course->id, 'valid' => 1);
$alternates = $DB->get_records_menu('block_bclnemail_alternate', $alt_params, '', 'id, address');

$blockname = bclnemail::_s('pluginname');
$header = bclnemail::_s('email');

$PAGE->set_context($context);
$PAGE->set_course($course);
$PAGE->navbar->add($blockname);
$PAGE->navbar->add($header);
$PAGE->set_title($blockname . ': ' . $header);
$PAGE->set_heading($blockname . ': ' . $header);
$PAGE->set_url('/blocks/bclnemail/email.php', array('courseid' => $courseid));
$PAGE->set_pagetype(bclnemail::PAGE_TYPE);
$PAGE->set_pagelayout('standard');

$PAGE->requires->jquery();
$PAGE->requires->js('/blocks/bclnemail/js/selection.js');

$course_roles = get_roles_used_in_context($context);

$filter_roles = $DB->get_records_select('role', sprintf('id IN (%s)', $config['roleselection']));

$roles = bclnemail::filter_roles($course_roles, $filter_roles);

$allgroups = groups_get_all_groups($courseid);

$mastercap = true;
$groups = $allgroups;

$restricted_view = (
    !has_capability('moodle/site:accessallgroups', $context) and
    $config['ferpa'] == 'strictferpa'
);

$respected_view = (
    !has_capability('moodle/site:accessallgroups', $context) and
    $course->groupmode == 1 and
    $config['ferpa'] == 'courseferpa'
);
if ($restricted_view || $respected_view) {
    $mastercap = false;
    $mygroups = groups_get_user_groups($courseid);
    $gids = implode(',', array_values($mygroups['0']));
    $groups = empty($gids) ?
            array() :
            $DB->get_records_select('groups', 'id IN (' . $gids . ')');
}

$globalaccess = empty($allgroups);

// Fill the course users by
$users = array();
$users_to_roles = array();
$users_to_groups = array();

$everyone = bclnemail::get_non_suspended_users($context, $courseid);

if (empty($config['messenger'])) {
    $msg = '0';
} else {
   $msg =   $config['messenger'];
}
foreach ($everyone as $userid => $user) {
    $usergroups = groups_get_user_groups($courseid, $userid);

    $gids = ($globalaccess or $mastercap) ?
        array_values($usergroups['0']) :
        array_intersect(array_values($mygroups['0']), array_values($usergroups['0']));

    $userroles = get_user_roles($context, $userid);
    $filterd = bclnemail::filter_roles($userroles, $roles);

    // Available groups
    if ((!$globalaccess and !$mastercap) and
        empty($gids) or empty($filterd) or $userid == $USER->id)
        continue;

    $groupmapper = function($id) use ($allgroups) { return $allgroups[$id]; };

    $users_to_groups[$userid] = array_map($groupmapper, $gids);
    $users_to_roles[$userid] = $filterd;
    if (!$user->suspended) {
        $users[$userid] = $user;
    }
}

if (empty($users)) {
    print_error('no_usergroups', 'block_bclnemail');
}

// we are presenting the form with values populated from either the log or drafts table in the db
if (!empty($type)) {
    
    $email = $DB->get_record('block_bclnemail_' . $type, array('id' => $typeid));
    //$emailmailto = array();
    if ($messageIDresend == 1) {
        list($email->mailto, $email->additional_emails) = bclnemail::clean($email->failuserids);
    }
} else {
    $email = new stdClass;
    $email->id = null;
}
$email->messageformat =  editors_get_preferred_format();
$default_sigid = $DB->get_field('block_bclnemail_signatures', 'id', array(
    'userid' => $USER->id, 'default_flag' => 1
));
$email->sigid = $default_sigid ? $default_sigid : -1;

// Some setters for the form
$email->type = $type;
$email->typeid = $typeid;

$editor_options = array(
    'trusttext' => false,
    'subdirs' => 1,
    'maxfiles' => EDITOR_UNLIMITED_FILES,
    'accepted_types' => '*',
    'context' => $context
);

$email = file_prepare_standard_editor(
    $email, 'message', $editor_options, $context, 'block_bclnemail', $type, $email->id
);

$selected = array();
if (!empty($email->mailto)) {
    foreach (explode(',', $email->mailto) as $id) {
        $selected[$id] = (object) array('id'=>$id,'firstname'=>null,'lastname'=>null,'email'=>$id,'mailformat'=>'1','suspended'=>'0','maildisplay'=>'2','status'=>'0'); 
        if(is_numeric($selected[$id]->id)) {
            $selected[$id] = $users[$id];
        } 
        unset($users[$id]);
    }
}


$form = new email_form(null, array(
    'editor_options' => $editor_options,
    'selected' => $selected,
    'users' => $users,
    'roles' => $roles,
    'groups' => $groups,
    'users_to_roles' => $users_to_roles,
    'users_to_groups' => $users_to_groups,
    'sigs' => array_map(function($sig) { return $sig->title; }, $sigs),
    'alternates' => $alternates
));

$warnings = array();
//

if ($form->is_cancelled()) {
    redirect(new moodle_url('/course/view.php?id=' . $courseid));
    // DWE we should check if we have selected users or emails around here. 
} else if ($data = $form->get_data()) {

    if (empty($data->subject)) {
        $warnings[] = get_string('no_subject', 'block_bclnemail');
    }
    if (empty($data->mailto) && empty($data->additional_emails)) {
        $warnings[] = get_string('no_users', 'block_bclnemail');
    }
    if (empty($warnings)) {
        // Submitted data //////////////////////////////////////////////////////
        $data->time = time();
        $data->format = $data->message_editor['format'];
        $data->message = $data->message_editor['text'];
        $data->messageWithSigAndAttach = "";
        // Store data; id is needed for file storage ///////////////////////////
        if (isset($data->send)) {
            $data->id = $DB->insert_record('block_bclnemail_log', $data);
            $table = 'log';
        } else if (isset($data->draft)) {
            $table = 'drafts';

            if (!empty($typeid) and $type == 'drafts') {
                $data->id = $typeid;
                $DB->update_record('block_bclnemail_drafts', $data);
            } else {
                $data->id = $DB->insert_record('block_bclnemail_drafts', $data);
            }
        }
        $data = file_postupdate_standard_editor(
            $data, 'message', $editor_options, $context, 'block_bclnemail', $table, $data->id
        );
        $DB->update_record('block_bclnemail_' . $table, $data);

        $prepender = $config['prepend_class'];
        if (!empty($data->subject)) {
            $subject = $data->subject;
        } else {
            $subject = "[{$course->$prepender}] $data->subject";
        }

        // An instance id is needed before storing the file repository /////////

        // Send emails /////////////////////////////////////////////////////////
        if (isset($data->send)) {
            if ($type == 'drafts') {
                bclnemail::draft_cleanup($context->id, $typeid);
            }
            // deal with possible signature, will be appended to message in a little bit.
            if (!empty($sigs) and $data->sigid > -1) {
                $sig = $sigs[$data->sigid];

                $signaturetext = file_rewrite_pluginfile_urls($sig->signature, 'pluginfile.php', $context->id, 'block_bclnemail', 'signature', $sig->id, $editor_options);

                
            }


            if(empty($signaturetext)){
                $data->messageWithSigAndAttach = $data->message;
            }
            else{
                if($data->format == 0 || $data->format == 2 ){
                    $data->messageWithSigAndAttach = $data->message . "\n\n" .$signaturetext;
                }else{
                    $data->messageWithSigAndAttach = $data->message . "<br /> <br /> <p></p>" .$signaturetext;
                }
            }

                // Prepare html content of message
            $data->message = file_rewrite_pluginfile_urls($data->message, 'pluginfile.php', $context->id, 'block_bclnemail', $table, $data->id, $editor_options);


            // Same user, alternate email //////////////////////////////////////
            if (!empty($data->alternateid)) {
                $user = clone($USER);
                $user->email = $alternates[$data->alternateid];
            } else {
                $user = $USER;
            }
            $data->failuserids = array();
            // DWE -> Begin hopefully new way of dealing with messagetext and messagehtml

            // TEXT
            // This is where we'll need to factor in the preferences of the receiver.
            $messagetext = format_text_email($data->messageWithSigAndAttach, $data->format);

            // HTML
            $messagehtml = format_text($data->messageWithSigAndAttach, $data->format);
            
            $messagetext_base = '';
            $messagehtml_base = ''; 
            $from = $DB->get_record('user',array('id' =>$USER->id));

            if(!empty($data->mailto)) {
                foreach (explode(',', $data->mailto) as $userid) {
                    // Email gets sent here
                    // if progress report selected generate it here
                    //
  
                    if ($data->progress) {
                       $params23 = array($userid);
                       $result_student = $DB->get_records_sql("select lastname, firstname from {user} u where u.id = ?",$params23);
                       foreach($result_student as $student) {
                          $nm = $student->firstname.' '.$student->lastname;
                       }   
                       $base = 'Student: '.$nm; 
                       $params = array($courseid,$userid);
                       $result_last = $DB->get_records_sql("select * from {user_lastaccess} where courseid=? and userid=? order by timeaccess",$params);
                       $ltime = "N/A";
                       foreach($result_last as $last) {
                       	 $lt=$last->timeaccess;
                       	 $ltime = date("Y-m-d",$lt);
                       }		
  
                       $params21 = array($courseid);
                       $result_context = $DB->get_records_sql("select id from {context} where contextlevel <= 50 and instanceid = ? order by contextlevel desc",$params21);
                       foreach ($result_context as $ctx) {
             	           $cid = $ctx->id;
             	         }  
             	         $params22 = array($cid);
             	         $tl = 'N/A';
             	         $tcnt = 0;
                       $result_teacher = $DB->get_records_sql("select lastname,firstname from {role_assignments} r,{user} u where u.id = r.userid and roleid = 3  and contextid = ?",$params22);
      
//             	         $result_teacher = $DB->get_records_sql("select userid,firstname,lastname from mdl_role_assignments ra, mdl_role r,mdl_user mus where r.shortname='teacher' and r.id=ra.roleid and mus.id = ra.userid and contextid=? order by lastname",$params22);
                       foreach ($result_teacher as $tlist) {
                           $tcnt = $tcnt + 1;
                           $tn = $tlist->firstname .' '.$tlist->lastname;
                           if ($tcnt == 1) {
                           	   $tl = $tn;
                           } else {
                           	   $tl = $tl.'<br>'.$tn;
                           }
                       }    
                       $result_course = $DB->get_records_sql("select * from {course} where id=?",$params21);
                       foreach ($result_course as $crs) {
                       	   $cn = $crs->fullname;
                       }
                       $base = $base .'<br> Course: '.$cn;
                       	   		                 	         
                       $base = $base .'<br> Teacher(s): '.$tl;
                       $rdate = date("Y-m-d");
                       $base = $base .'<br> Report Date: '.$rdate;
                       $base = $base .'<br> Last time in Course: '.$ltime;
                       if ($data->cgrade) {
                       	  $gtot = 0;
                       	  $result_citem = $DB->get_records_sql("select * from {grade_items} where courseid =? and itemtype = 'course'",$params21);
                          foreach ($result_citem as $citem) {
                             $mdlid = $citem->id;
                             $maxgrade = $citem->grademax;
                   	         $params221 = array($courseid,$userid,$mdlid);
                             $result_final = $DB->get_records_sql("select finalgrade from {grade_grades} gg,{grade_items} gi 
                                     where gi.id = gg.itemid and courseid=? and userid=? 
                                     and gi.id=?  and gg.excluded = 0 ",$params221);
                             foreach ($result_final as $cfinal) {
                                  $gtot = $cfinal->finalgrade;
                                  if ($maxgrade > 100) {
                                      $gtot = $gtot / $maxgrade;
                                      $gtot = $gtot * 100;
                                  } 
                                  $gtot = round($gtot,1); 
                             }     
                             $base = $base .'<br> Current Grade in Course: '.$gtot .' %';
	                        }
	                        
                       }
                       if ($data->pcomp) {
                           $rcomp = $DB->count_records_sql("select COUNT(*) FROM {course_completion_criteria} where course =?",$params21);
                           if ($rcomp >= 1) {
                           	  $params223 = array($courseid,$userid);
                              $result_comp = $DB->count_records_sql("select COUNT(*) FROM {course_completion_crit_compl} where course =?  and userid=?",$params223);
                              $rval = ($result_comp / $rcomp) * 100;
                              $rval = round($rval,1); 
                              $base = $base .'<br> Portion of Course Completed: '.$rval .' %';
	                         }    
                       }
                       $base = $base .'<br><table border="1"><th><b>Activity Name</b></th><th><b>Category </b></th><th><b>Grade</b></th><th><b>Date Submitted</b></th></tr>';
                       $params2 = array($courseid);
                       $result = $DB->get_records_sql("select * from {course_sections} where course =?",$params2);
                       $ln = '';
                       //echo 'Course id '.$courseid .' user id '.$userid;
                       foreach($result as $seq) {
                       	   $seqarr = $seq->sequence;
                       	   
                       	   $wordChunks = explode(",", $seqarr);
               
                       	   for($i = 0; $i < count($wordChunks); $i++){
                       	   	   $sec = $wordChunks[$i];
                       	   	   $params33 = array($sec);
                       	   	   //$params33 = array($sec,$courseid);
                       	   	   
                       	   	   //echo '<br> mod '.$sec .'<br>';
                       	   	   $result_mod = $DB->get_records_sql("select * from {course_modules} where id = ?",$params33);
	                             //$result_mod = $DB->get_records_sql("select * from {course_modules} where course = ? and id=?",$params33);
	                            
	                             foreach($result_mod as $mlist) {
	                             	  $mtype = $mlist->module;
	                             	  $modid = $mlist->instance;
	                             	  //echo '<br> instance '.$modid .' mtype '.$mtype;
	                                //if (($mtype == 16) or ($mtype == 1)){
	                             	  	  $params44 = array($courseid,$modid);
	                             	  	  
	                             	  //	  $result5 = $DB->get_records_sql("select * from {grade_items} where iteminstance=?",$params44);   
	                                  	  $result5 = $DB->get_records_sql("select * from {grade_items} where courseid = ? and iteminstance=?",$params44);   
	                                    
    	                                foreach ($result5 as $item) {
             	                           $catid = $item->categoryid;
             	                           //echo 'catorgy is '.$catid;
             	                           if (is_null($catid)) {
             	                           }	else {
             	                              $params2 = array($catid);
                                            $result_cat = $DB->get_records_sql("select * from {grade_categories} where  id=? ",$params2); 
                                            foreach ($result_cat as $cats) {
             	                                $cn = trim($cats->fullname);
             	            	                }    
             	            	                $lc = strlen($cn);
             	            	                if ($lc > 3) {
             	                                 $act = $item->itemname;
                                               $mdlid= $item->id;
                                               $mname = $item->itemmodule;
			                                         $mgrade = $item->grademax;
				                                       $mgrade = round($mgrade,2);
                          	                   $ln=$ln.'<tr><td>'.$act.'</td><td>'.$cn.'</td>';
                          	                 
                  	                           $params3 = array($courseid,$userid,$mdlid);
                  	                           $resultgrade = $DB->get_records_sql("select finalgrade, g.excluded, g.timemodified from {grade_grades} g,{grade_items} gi where gi.id = g.itemid and courseid=? and userid=? and gi.id=? ",$params3);
                                               $cnt = '0';
                                               $grade = 'N/A';
                                               $dmark = 'N/A';
                                               
                                             
                                               foreach ($resultgrade as $gid) {
                            	                   $cnt = '1';
                      	                         $sgrade = $gid->finalgrade;
                      	                         $exclude = $gid->excluded;
                      	                         if ($exclude > 0) {
                                                 	  $grade = "OMIT";
                                                 	  $dmark = "&nbsp;&nbsp;&nbsp;";
                      	                         } else {	
                      	                            if (is_null($sgrade)) {
			 	                                               $dmark = "N/A";
				                                               $grade = "N/A";
				                                            } else {
			 	                                               $sgrade = round($sgrade,2);
					                                             $grade = $sgrade . "/" . $mgrade;
                                                       $dmark = $gid->timemodified;
                                                       $dmark = strftime("%Y-%m-%d", $dmark);
                                                    }
                                                 }         
 				                                         
                                               }
                                               $ln = $ln.'<td>'.$grade.'</td><td>'.$dmark.'</td></tr>';
                                            } 
                                         }
                                      }
                               
                                  //}
                               
                               }
                           
                           }
                                                       
                       }
                       $base = $base.$ln;
                       $base = $base.'</table>';
                       //echo $base;
                       //exit();
                       $messagetext = $messagetext.'  '.$base;
                       $messagehtml = $messagehtml.'<br>'.$base;
                    //   echo $messagehtml;
                    //   exit();
                    }
                    if ($msg == '1') {                     
                       $success = email_to_user($everyone[$userid], $user, $subject,$messagetext, $messagehtml);
                    } else {
                       $userto = $DB->get_record('user',array('id' =>$userid));
                       $message = new \core\message\message();
                       $message->component = 'moodle';
                       $message->name = 'instantmessage';
                       $message->userfrom = $from;
                       $message->userto = $userto ;
                       $message->subject = $subject;
                       $message->fullmessage = $messagehtml;
                       $message->fullmessageformat = FORMAT_MARKDOWN;
                       $message->fullmessagehtml = $messagehtml;
                       $message->smallmessage = $messagehtml;
                       $message->notification = '0';
                       $message->replyto = $user->email;
                       $message->courseid = SITEID;
                       $success = message_send($message);
                    }
                    //
                    // Get the additional emails
                    //
                $params = array($userid);
                $result_field1 = $DB->get_records_sql("select * from {user_info_field} where shortname IN ('Contact1','Contact2','Contact3')",$params);
                $i = 0;
                foreach($result_field1 as $field1) {
                   $fid=$field1->id;
                   $params2 = array($userid,$fid);
                   $result_data1 = $DB->get_records_sql("select * from {user_info_data} where userid=? and fieldid = ?",$params2);
                   foreach($result_data1 as $data1) {
                      $em = trim($data1->data);
                      $sl = strlen($em);
                      if ($sl > 3) {
                          $fakeuser = $DB->get_record('user', array('id' => $userid), '*', MUST_EXIST);
                          $fakeuser->id = 99999900 + $i;
                          $fakeuser->email = $em;
                          $fakeuser->lastname ='Contact';
                          $fakeuser->firtname = 'Student';
                          $fakeuser->mailformat = 1;
                          $additional_email_success = email_to_user($fakeuser, $user, $subject, $messagetext, $messagehtml);
                          if (!$additional_email_success) {
                              $data->failuserids[] = $additional_email;
                              // will need to notify that an email is incorrect
                              $warnings[] = get_string("no_email_address", 'block_bclnemail', $fakeuser->email);
                          }
                      }
                   }
                }                  	
                    //exit();
                if (!$success) {
                    $warnings[] = get_string("no_email", 'block_bclnemail', $everyone[$userid]);
                    $data->failuserids[] = $userid;
                }
                if ($data->receipt) {
      	            $messagetext_base = $messagetext_base.'\n'.$messagetext;
                    $messagehtml_base = $messagehtml_base.'<br>'.$messagehtml;  

                }
                $messagetext = '';
                $messagehtml = '';
                $messagetext = format_text_email($data->messageWithSigAndAttach, $data->format);

            // HTML
                $messagehtml = format_text($data->messageWithSigAndAttach, $data->format);
            
            }
          }  
//exit();
        }
    }
    $email = $data;
}

 if (isset($data->receipt)) {
 	   if ($data->receipt) {
         email_to_user($USER, $user, $subject, $messagetext_base, $messagehtml_base);
     }
 }              

$form->set_data($email);
if (empty($warnings)) {
    if (isset($email->send)) {
        redirect(new moodle_url('/blocks/bclnemail/emaillog.php', array('courseid' => $course->id)));
    } else if (isset($email->draft)) {
        $warnings['success'] = get_string("changessaved");
    }
}
//exit();
echo $OUTPUT->header();
echo $OUTPUT->heading($blockname);

foreach ($warnings as $type => $warning) {
    $class = ($type === 'success') ? 'notifysuccess' : 'notifyproblem';
    echo $OUTPUT->notification($warning, $class);
}

echo html_writer::start_tag('div', array('class' => 'no-overflow'));
$form->display();

echo html_writer::end_tag('div');
echo '<table align="center"><tr><td><Small>Note: This email will be sent to the student (using student email) as well as any contacts that have been identified in the </td></tr>
   <tr><td>User Profile Fields (email addresses in any of Contact1, Contact2, or Contact3). &nbsp;&nbsp;See block instructions.txt in the installation files for more info.
</small></td></tr></table>';
echo $OUTPUT->footer();
